unit DBObjects;

interface

uses classes, db, dbtables, HashMap, DCL_intf;

type

//-----------------------------------------------------------------

  TValue = class(TObject)
  public
	Value : Variant;
	constructor Create(const Value : Variant);
	destructor Destroy; override;
  end;

//-----------------------------------------------------------------

  TFieldSet = class(TStrHashMap)
  public
	constructor Create(const FieldName: string; const Value : Variant); overload;
	constructor Create; overload;
	procedure Add(const FieldName: string; const Value : Variant);
  end;

//-----------------------------------------------------------------

  TSimpleDBObject = class(TObject)
  public
	procedure SetFieldValue(FieldName : String; Value : Variant);
	function GetFieldValue(FieldName : String) : Variant;
	procedure Remove;
	procedure Save;
	procedure Load(const Key : TFieldSet);
	constructor Create(Table: TTable; const PrimaryFieldNames : string;
		const Key : TFieldSet = nil);
	destructor Destroy; override;
  protected
	Fields : TFieldSet;
	PrimaryKeyNames : TStringList;
	Table : TTable;
  private
  	procedure DoRefresh;
  end;

//-----------------------------------------------------------------
function StringValue(const Value : Variant) : String;
function IntegerValue(const Value : Variant) : Integer;
//-----------------------------------------------------------------

implementation

uses Sysutils;

//-----------------------------------------------------------------
{ TValue }
//-----------------------------------------------------------------

constructor TValue.Create(const Value: Variant);
begin
	self.Value := Value;
end;

//-----------------------------------------------------------------

destructor TValue.Destroy;
begin
	inherited;
end;

//-----------------------------------------------------------------
{ TFieldSet }
//-----------------------------------------------------------------

constructor TFieldSet.Create(const FieldName: string; const Value : Variant);
begin
	inherited Create;
	Add(FieldName, Value);
end;

//-----------------------------------------------------------------

procedure TFieldSet.Add(const FieldName: string; const Value: Variant);
begin
	PutValue(FieldName, TValue.Create(Value));
end;

//-----------------------------------------------------------------

constructor TFieldSet.Create;
begin
	inherited Create;
end;

//-----------------------------------------------------------------
{ TSimpleDBObject }
//-----------------------------------------------------------------

constructor TSimpleDBObject.Create(Table: TTable; const PrimaryFieldNames : string;
	const Key : TFieldSet = nil);
var i : Integer;
	DelimPos : Integer;
	FieldName, FieldNames : string;
begin
	self.Table := Table;

	PrimaryKeyNames := TStringList.Create;

	//    
	Fields := TFieldSet.Create;
	for i:=0 to Table.FieldDefs.Count-1 do
		Fields.PutValue(UpperCase(Table.FieldDefs[i].Name), TValue.Create(null));

	//   ,   
//	Table.IndexDefs.Update;
	FieldNames := UpperCase(PrimaryFieldNames);
	repeat
		DelimPos := Pos(';',FieldNames);
		if DelimPos<>0 then
		begin
			FieldName := Copy(FieldNames,1,DelimPos-1);
			Delete(FieldNames,1,DelimPos);
		end else
			FieldName := FieldNames;

		if (Fields.ContainsKey(FieldName)) then
			PrimaryKeyNames.Add(FieldName);
	until DelimPos=0;

	if Key<>nil then Load(Key);
end;

//-----------------------------------------------------------------

destructor TSimpleDBObject.Destroy;
begin
	Fields.Free;
	PrimaryKeyNames.Free;
	inherited Destroy;
end;

//-----------------------------------------------------------------

function TSimpleDBObject.GetFieldValue(FieldName: String): Variant;
begin
	FieldName := UpperCase(FieldName);
	Result := TValue(Fields.GetValue(FieldName)).Value;
end;

//-----------------------------------------------------------------

procedure TSimpleDBObject.Load(const Key: TFieldSet);
var TableActive : Boolean;
	i: IStrIterator;
	KeyFields, KeyName : string;
	KeyValues : array of Variant;
begin
	TableActive := Table.Active;
	SetLength(KeyValues, Key.Size);

	i := Key.KeySet.First;
	while i.HasNext do
	begin
		KeyName := i.Next;
		KeyFields := KeyFields+KeyName;
		if i.HasNext then KeyFields := KeyFields+';';
		KeyValues[i.PreviousIndex] := TValue(Key.GetValue(KeyName)).Value;
	end;

	Table.Active := True;
	if (Table.Locate(KeyFields, VarArrayOf(KeyValues), [])) then
	begin
		DoRefresh;
	end;
	Table.Active := TableActive;
end;

//-----------------------------------------------------------------

procedure TSimpleDBObject.Save;
var TableActive : Boolean;
	i : IStrIterator;
	j : Integer;
	KeyFields, KeyName : string;
	KeyValues : array of Variant;
	ToUpdate : Boolean;
begin
	TableActive := Table.Active;
	SetLength(KeyValues, PrimaryKeyNames.Count);

	for j:=0 to PrimaryKeyNames.Count-1 do
	begin
		KeyName := PrimaryKeyNames[j];
		KeyFields := KeyFields+KeyName;
		if j<>PrimaryKeyNames.Count-1 then KeyFields := KeyFields+';';
		KeyValues[j] := TValue(Fields.GetValue(KeyName)).Value;
	end;

	Table.Close;
	Table.Open;

	ToUpdate := Table.Locate(KeyFields, VarArrayOf(KeyValues), []);

	if ToUpdate then Table.Edit
	else Table.Append;

	i := Fields.KeySet.First;
	while i.HasNext do
	begin
		KeyName := i.Next;
		Table.FieldValues[KeyName] := TValue(Fields.GetValue(KeyName)).Value;
	end;
	Table.Post;
	Table.Refresh;

//	if not ToUpdate then Table.Next; // ???

	DoRefresh;

	Table.Active := TableActive;
end;

//-----------------------------------------------------------------

procedure TSimpleDBObject.Remove;
var TableActive : Boolean;
	j : Integer;
	KeyFields, KeyName : string;
	KeyValues : array of Variant;
begin
	TableActive := Table.Active;
	SetLength(KeyValues, PrimaryKeyNames.Count);

	for j:=0 to PrimaryKeyNames.Count-1 do
	begin
		KeyName := PrimaryKeyNames[j];
		KeyFields := KeyFields+KeyName;
		if j<>PrimaryKeyNames.Count-1 then KeyFields := KeyFields+';';
		KeyValues[j] := TValue(Fields.GetValue(KeyName)).Value;
	end;

	Table.Close;
	Table.Open;
	if (Table.Locate(KeyFields, VarArrayOf(KeyValues), [])) then
		Table.Delete;
	Table.Refresh;

	Table.Active := TableActive;
end;

//-----------------------------------------------------------------

procedure TSimpleDBObject.SetFieldValue(FieldName: String;
  Value: Variant);
begin
	FieldName := UpperCase(FieldName);
	TValue(Fields.GetValue(FieldName)).Value := Value;
end;

//-----------------------------------------------------------------
//-----------------------------------------------------------------
//-----------------------------------------------------------------

function StringValue(const Value : Variant) : String;
begin
	if Value=null then Result := ''
	else Result := Value;
end;

//-----------------------------------------------------------------

function IntegerValue(const Value : Variant) : Integer;
begin
	if Value=null then Result := 0
	else Result := Value;
end;

//-----------------------------------------------------------------

procedure TSimpleDBObject.DoRefresh;
var	i: IStrIterator;
	KeyName : string;
begin
	i := Fields.KeySet.First;
	while i.HasNext do
	begin
		KeyName := i.Next;
		TValue(Fields.GetValue(KeyName)).Value := Table.FieldValues[KeyName];
	end;
end;

//-----------------------------------------------------------------
end.
